@@ -1,8 +1,8 @@
 /*
    8bit strings utilities
 
-   Copyright (C) 2007, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 2007-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Rostislav Benes, 2007
@@ -198,7 +198,7 @@ str_8bit_length2 (const char *text, int size)
 }
 
 static gchar *
-str_8bit_conv_gerror_message (GError * error, const char *def_msg)
+str_8bit_conv_gerror_message (GError * mcerror, const char *def_msg)
 {
     GIConv conv;
     gchar *ret;
@@ -214,7 +214,7 @@ str_8bit_conv_gerror_message (GError * error, const char *def_msg)
 
         buf = g_string_new ("");
 
-        if (str_convert (conv, error->message, buf) != ESTR_FAILURE)
+        if (str_convert (conv, mcerror->message, buf) != ESTR_FAILURE)
             ret = g_string_free (buf, FALSE);
         else
         {
@@ -338,6 +338,8 @@ str_8bit_fit_to_term (const char *text, int width, align_crt_t just_mode)
     }
 
   finally:
+    if (actual >= result + sizeof (result))
+        actual = result + sizeof (result) - 1;
     actual[0] = '\0';
     return result;
 }
