@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2007-2014
+   Copyright (C) 2007-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -573,20 +573,20 @@ dview_get_byte (char *str, gboolean * result)
 
 /* --------------------------------------------------------------------------------------------- */
 
+#ifdef HAVE_CHARSET
 /**
  * Get utf multibyte char from string
  *
- * @param char * str, int * char_width, gboolean * result
+ * @param char * str, int * char_length, gboolean * result
  * @return int as utf character or 0 and result == FALSE if fail
  */
 
 static int
-dview_get_utf (char *str, int *char_width, gboolean * result)
+dview_get_utf (char *str, int *char_length, gboolean * result)
 {
     int res = -1;
     gunichar ch;
-    gchar *next_ch = NULL;
-    int width = 0;
+    int ch_len = 0;
 
     *result = TRUE;
 
@@ -602,15 +602,14 @@ dview_get_utf (char *str, int *char_width, gboolean * result)
         ch = *str;
     else
     {
+        gchar *next_ch;
+
         ch = res;
-        /* Calculate UTF-8 char width */
+        /* Calculate UTF-8 char length */
         next_ch = g_utf8_next_char (str);
-        if (next_ch != NULL)
-            width = next_ch - str;
-        else
-            ch = 0;
+        ch_len = next_ch - str;
     }
-    *char_width = width;
+    *char_length = ch_len;
     return ch;
 }
 
@@ -648,6 +647,7 @@ dview_str_utf8_offset_to_pos (const char *text, size_t length)
     }
     return max (length, (size_t) result);
 }
+#endif /*HAVE_CHARSET */
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -1413,15 +1413,17 @@ cvt_mget (const char *src, size_t srcsize, char *dst, int dstsize, int skip, int
             }
             else if (skip > 0)
             {
-                int utf_ch = 0;
+#ifdef HAVE_CHARSET
                 gboolean res;
-                int w;
+                int ch_len = 1;
+
+                (void) dview_get_utf ((char *) src, &ch_len, &res);
+
+                if (ch_len > 1)
+                    skip += ch_len - 1;
+#endif
 
                 skip--;
-                utf_ch = dview_get_utf ((char *) src, &w, &res);
-                if (w > 1)
-                    skip += w - 1;
-                (void) utf_ch;
             }
             else
             {
@@ -1514,15 +1516,16 @@ cvt_mgeta (const char *src, size_t srcsize, char *dst, int dstsize, int skip, in
             }
             else if (skip != 0)
             {
-                int utf_ch = 0;
+#ifdef HAVE_CHARSET
                 gboolean res;
-                int w;
+                int ch_len = 1;
+
+                (void) dview_get_utf ((char *) src, &ch_len, &res);
+                if (ch_len > 1)
+                    skip += ch_len - 1;
+#endif
 
                 skip--;
-                utf_ch = dview_get_utf ((char *) src, &w, &res);
-                if (w > 1)
-                    skip += w - 1;
-                (void) utf_ch;
             }
             else
             {
@@ -2431,8 +2434,8 @@ dview_init (WDiff * dview, const char *args, const char *file1, const char *file
     dview->merged[DIFF_RIGHT] = FALSE;
     dview->hdiff = NULL;
     dview->dsrc = dsrc;
-    dview->converter = str_cnv_from_term;
 #ifdef HAVE_CHARSET
+    dview->converter = str_cnv_from_term;
     dview_set_codeset (dview);
 #endif
     dview->a[DIFF_LEFT] = g_array_new (FALSE, FALSE, sizeof (DIFFLN));
@@ -2490,8 +2493,10 @@ dview_fini (WDiff * dview)
         f_close (dview->f[DIFF_LEFT]);
     }
 
+#if HAVE_CHARSET
     if (dview->converter != str_cnv_from_term)
         str_close_conv (dview->converter);
+#endif
 
     destroy_hdiff (dview);
     if (dview->a[DIFF_LEFT] != NULL)
@@ -2592,9 +2597,11 @@ dview_display_file (const WDiff * dview, diff_place_t ord, int r, int c, int hei
                 {
                     char att[BUFSIZ];
 
+#ifdef HAVE_CHARSET
                     if (dview->utf8)
                         k = dview_str_utf8_offset_to_pos (p->p, width);
                     else
+#endif
                         k = width;
 
                     cvt_mgeta (p->p, p->u.len, buf, k, skip, tab_size, show_cr,
@@ -2604,18 +2611,21 @@ dview_display_file (const WDiff * dview, diff_place_t ord, int r, int c, int hei
 
                     for (cnt = 0; cnt < strlen (buf) && col < width; cnt++)
                     {
-                        int w;
                         gboolean ch_res;
 
+#ifdef HAVE_CHARSET
                         if (dview->utf8)
                         {
-                            next_ch = dview_get_utf (buf + cnt, &w, &ch_res);
-                            if (w > 1)
-                                cnt += w - 1;
+                            int ch_len;
+
+                            next_ch = dview_get_utf (buf + cnt, &ch_len, &ch_res);
+                            if (ch_len > 1)
+                                cnt += ch_len - 1;
                             if (!g_unichar_isprint (next_ch))
                                 next_ch = '.';
                         }
                         else
+#endif
                             next_ch = dview_get_byte (buf + cnt, &ch_res);
 
                         if (ch_res)
@@ -2646,9 +2656,11 @@ dview_display_file (const WDiff * dview, diff_place_t ord, int r, int c, int hei
                 if (ch == CHG_CH)
                     tty_setcolor (DFF_CHH_COLOR);
 
+#ifdef HAVE_CHARSET
                 if (dview->utf8)
                     k = dview_str_utf8_offset_to_pos (p->p, width);
                 else
+#endif
                     k = width;
                 cvt_mget (p->p, p->u.len, buf, k, skip, tab_size, show_cr);
             }
@@ -2676,18 +2688,21 @@ dview_display_file (const WDiff * dview, diff_place_t ord, int r, int c, int hei
         col = 0;
         for (cnt = 0; cnt < strlen (buf) && col < width; cnt++)
         {
-            int w;
             gboolean ch_res;
 
+#ifdef HAVE_CHARSET
             if (dview->utf8)
             {
-                next_ch = dview_get_utf (buf + cnt, &w, &ch_res);
-                if (w > 1)
-                    cnt += w - 1;
+                int ch_len;
+
+                next_ch = dview_get_utf (buf + cnt, &ch_len, &ch_res);
+                if (ch_len > 1)
+                    cnt += ch_len - 1;
                 if (!g_unichar_isprint (next_ch))
                     next_ch = '.';
             }
             else
+#endif
                 next_ch = dview_get_byte (buf + cnt, &ch_res);
             if (ch_res)
             {
