@@ -2,7 +2,7 @@
    Routines invoked by a function key
    They normally operate on the current panel.
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -444,7 +444,7 @@ nice_cd (const char *text, const char *xtext, const char *help,
 
     g_free (machine);
 
-    if (*cd_path != PATH_SEP)
+    if (!IS_PATH_SEP (*cd_path))
     {
         char *tmp = cd_path;
 
@@ -863,7 +863,7 @@ mkdir_cmd (void)
     {
         vfs_path_t *absdir;
 
-        if (dir[0] == '/' || dir[0] == '~')
+        if (IS_PATH_SEP (dir[0]) || dir[0] == '~')
             absdir = vfs_path_from_str (dir);
         else
         {
@@ -1515,7 +1515,7 @@ single_dirsize_cmd (void)
         p = vfs_path_from_str (entry->fname);
 
         memset (&dsm, 0, sizeof (dsm));
-        status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 1.0, dirsize_status_init_cb,
+        status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 0, dirsize_status_init_cb,
                          dirsize_status_update_cb, dirsize_status_deinit_cb);
 
         if (compute_dir_size (p, &dsm, &dir_count, &count, &total, TRUE) == FILE_CONT)
@@ -1550,7 +1550,7 @@ dirsizes_cmd (void)
     dirsize_status_msg_t dsm;
 
     memset (&dsm, 0, sizeof (dsm));
-    status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 1.0, dirsize_status_init_cb,
+    status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 0, dirsize_status_init_cb,
                      dirsize_status_update_cb, dirsize_status_deinit_cb);
 
     for (i = 0; i < panel->dir.len; i++)
