@@ -1,9 +1,8 @@
 /*
    File management.
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2011, 2012, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Janne Kukonlehto, 1994, 1995
@@ -12,7 +11,7 @@
    Jakub Jelinek, 1995, 1996
    Norbert Warmuth, 1997
    Pavel Machek, 1998
-   Andrew Borodin <aborodin@vmail.ru>, 2011, 2012, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2011-2014
 
    The copy code was based in GNU's cp, and was written by:
    Torbjorn Granlund, David MacKenzie, and Jim Meyering.
@@ -150,10 +149,10 @@ static const char *op_names1[] = {
  * %m - "with source mask" or question mark for delete
  * %s - source name (truncated)
  * %d - number of marked files
- * %e - "to:" or question mark for delete
- *
- * xgettext:no-c-format */
-static const char *one_format = N_("%o %f \"%s\"%m");
+ * %n - the '\n' symbol to form two-line prompt for delete or space for other operations
+ */
+/* xgettext:no-c-format */
+static const char *one_format = N_("%o %f%n\"%s\"%m");
 /* xgettext:no-c-format */
 static const char *many_format = N_("%o %d %f%m");
 
@@ -164,12 +163,9 @@ static const char *prompt_parts[] = {
     N_("directories"),
     N_("files/directories"),
     /* TRANSLATORS: keep leading space here to split words in Copy/Move dialog */
-    N_(" with source mask:"),
-    N_("to:")
+    N_(" with source mask:")
 };
 
-static const char *question_format = N_("%s?");
-
 /*** file scope variables ************************************************************************/
 
 /* the hard link cache */
@@ -192,9 +188,43 @@ static GSList *dest_dirs = NULL;
 
 static FileProgressStatus transform_error = FILE_CONT;
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
+static void
+dirsize_status_locate_buttons (dirsize_status_msg_t * dsm)
+{
+    status_msg_t *sm = STATUS_MSG (dsm);
+    Widget *wd = WIDGET (sm->dlg);
+    int y, x;
+
+    y = wd->y + 5;
+    x = wd->x;
+
+    if (!dsm->allow_skip)
+    {
+        /* single button: "Abort" */
+        x += (wd->cols - dsm->abort_button->cols) / 2;
+        widget_set_size (dsm->abort_button, y, x,
+                         dsm->abort_button->lines, dsm->abort_button->cols);
+    }
+    else
+    {
+        /* two buttons: "Abort" and "Skip" */
+        int cols;
+
+        cols = dsm->abort_button->cols + dsm->skip_button->cols + 1;
+        x += (wd->cols - cols) / 2;
+        widget_set_size (dsm->abort_button, y, x, dsm->abort_button->lines,
+                         dsm->abort_button->cols);
+        x += dsm->abort_button->cols + 1;
+        widget_set_size (dsm->skip_button, y, x, dsm->skip_button->lines, dsm->skip_button->cols);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 transform_source (file_op_context_t * ctx, const vfs_path_t * source_vpath)
 {
@@ -487,12 +517,15 @@ make_symlink (file_op_context_t * ctx, const char *src_path, const char *dst_pat
  */
 
 static FileProgressStatus
-do_compute_dir_size (const vfs_path_t * dirname_vpath, void *ui,
-                     compute_dir_size_callback cback, size_t * dir_count, size_t * ret_marked,
-                     uintmax_t * ret_total, gboolean compute_symlinks)
+do_compute_dir_size (const vfs_path_t * dirname_vpath, dirsize_status_msg_t * dsm,
+                     size_t * dir_count, size_t * ret_marked, uintmax_t * ret_total,
+                     gboolean compute_symlinks)
 {
-    static unsigned short int update_ui_count = 0;
+    static guint64 timestamp = 0;
+    /* update with 25 FPS rate */
+    static const guint64 delay = G_USEC_PER_SEC / 25;
 
+    status_msg_t *sm = STATUS_MSG (dsm);
     int res;
     struct stat s;
     DIR *dir;
@@ -533,24 +566,23 @@ do_compute_dir_size (const vfs_path_t * dirname_vpath, void *ui,
         if (res == 0)
         {
             if (S_ISDIR (s.st_mode))
-            {
                 ret =
-                    do_compute_dir_size (tmp_vpath, ui, cback, dir_count, ret_marked, ret_total,
+                    do_compute_dir_size (tmp_vpath, dsm, dir_count, ret_marked, ret_total,
                                          compute_symlinks);
-                if (ret == FILE_CONT)
-                    ret =
-                        (cback == NULL) ? FILE_CONT : cback (ui, tmp_vpath, *dir_count, *ret_total);
-            }
             else
             {
+                ret = FILE_CONT;
+
                 (*ret_marked)++;
                 *ret_total += (uintmax_t) s.st_size;
+            }
 
-                update_ui_count++;
-                if ((update_ui_count & 31) == 0)
-                    ret =
-                        (cback == NULL) ? FILE_CONT : cback (ui, dirname_vpath, *dir_count,
-                                                             *ret_total);
+            if (ret == FILE_CONT && sm->update != NULL && mc_time_elapsed (&timestamp, delay))
+            {
+                dsm->dirname_vpath = tmp_vpath;
+                dsm->dir_count = *dir_count;
+                dsm->total_size = *ret_total;
+                ret = (FileProgressStatus) sm->update (sm);
             }
         }
 
@@ -564,7 +596,7 @@ do_compute_dir_size (const vfs_path_t * dirname_vpath, void *ui,
 /* --------------------------------------------------------------------------------------------- */
 
 static FileProgressStatus
-progress_update_one (FileOpTotalContext * tctx, file_op_context_t * ctx, off_t add)
+progress_update_one (file_op_total_context_t * tctx, file_op_context_t * ctx, off_t add)
 {
     struct timeval tv_current;
     static struct timeval tv_start = { };
@@ -619,8 +651,7 @@ warn_same_file (const char *fmt, const char *a, const char *b)
     union
     {
         void *p;
-          FileProgressStatus (*f) (enum OperationMode, const char *fmt,
-                                   const char *a, const char *b);
+        FileProgressStatus (*f) (enum OperationMode, const char *fmt, const char *a, const char *b);
     } pntr;
 /* *INDENT-ON* */
 
@@ -670,10 +701,8 @@ real_do_file_error (enum OperationMode mode, const char *error)
 static FileProgressStatus
 real_query_recursive (file_op_context_t * ctx, enum OperationMode mode, const char *s)
 {
-
     if (ctx->recursive_result < RECURSIVE_ALWAYS)
     {
-
         const char *msg;
         char *text;
 
@@ -716,11 +745,14 @@ real_query_recursive (file_op_context_t * ctx, enum OperationMode mode, const ch
 static FileProgressStatus
 do_file_error (const char *str)
 {
+/* *INDENT-OFF* */
     union
     {
         void *p;
-          FileProgressStatus (*f) (enum OperationMode, const char *);
+        FileProgressStatus (*f) (enum OperationMode, const char *);
     } pntr;
+/* *INDENT-ON* */
+
     pntr.f = real_do_file_error;
 
     if (mc_global.we_are_background)
@@ -734,11 +766,14 @@ do_file_error (const char *str)
 static FileProgressStatus
 query_recursive (file_op_context_t * ctx, const char *s)
 {
+/* *INDENT-OFF* */
     union
     {
         void *p;
-          FileProgressStatus (*f) (file_op_context_t *, enum OperationMode, const char *);
+        FileProgressStatus (*f) (file_op_context_t *, enum OperationMode, const char *);
     } pntr;
+/* *INDENT-ON* */
+
     pntr.f = real_query_recursive;
 
     if (mc_global.we_are_background)
@@ -753,12 +788,15 @@ static FileProgressStatus
 query_replace (file_op_context_t * ctx, const char *destname, struct stat *_s_stat,
                struct stat *_d_stat)
 {
+/* *INDENT-OFF* */
     union
     {
         void *p;
-          FileProgressStatus (*f) (file_op_context_t *, enum OperationMode, const char *,
-                                   struct stat *, struct stat *);
+        FileProgressStatus (*f) (file_op_context_t *, enum OperationMode, const char *,
+                                 struct stat *, struct stat *);
     } pntr;
+/* *INDENT-ON* */
+
     pntr.f = file_progress_real_query_replace;
 
     if (mc_global.we_are_background)
@@ -819,7 +857,7 @@ files_error (const char *format, const char *file1, const char *file2)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-copy_file_file_display_progress (FileOpTotalContext * tctx, file_op_context_t * ctx,
+copy_file_file_display_progress (file_op_total_context_t * tctx, file_op_context_t * ctx,
                                  struct timeval tv_current, struct timeval tv_transfer_start,
                                  off_t file_size, off_t n_read_total)
 {
@@ -874,7 +912,8 @@ copy_file_file_display_progress (FileOpTotalContext * tctx, file_op_context_t *
 
 /* {{{ Move routines */
 static FileProgressStatus
-move_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s, const char *d)
+move_file_file (file_op_total_context_t * tctx, file_op_context_t * ctx, const char *s,
+                const char *d)
 {
     struct stat src_stats, dst_stats;
     FileProgressStatus return_status = FILE_CONT;
@@ -1024,7 +1063,7 @@ move_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *
 /** Don't update progress status if progress_count==NULL */
 
 static FileProgressStatus
-erase_file (FileOpTotalContext * tctx, file_op_context_t * ctx, const vfs_path_t * vpath)
+erase_file (file_op_total_context_t * tctx, file_op_context_t * ctx, const vfs_path_t * vpath)
 {
     struct stat buf;
 
@@ -1070,7 +1109,7 @@ erase_file (FileOpTotalContext * tctx, file_op_context_t * ctx, const vfs_path_t
   skipall->remove as much as possible
 */
 static FileProgressStatus
-recursive_erase (FileOpTotalContext * tctx, file_op_context_t * ctx, const vfs_path_t * vpath)
+recursive_erase (file_op_total_context_t * tctx, file_op_context_t * ctx, const vfs_path_t * vpath)
 {
     struct dirent *next;
     DIR *reading;
@@ -1234,12 +1273,11 @@ panel_get_file (WPanel * panel)
  */
 
 static FileProgressStatus
-panel_compute_totals (const WPanel * panel, void *ui, compute_dir_size_callback cback,
-                      size_t * ret_count, uintmax_t * ret_total, gboolean compute_symlinks)
+panel_compute_totals (const WPanel * panel, dirsize_status_msg_t * sm, size_t * ret_count,
+                      uintmax_t * ret_total, gboolean compute_symlinks)
 {
     int i;
-    size_t lc_count = 0;
-    uintmax_t lc_total = 0;
+    size_t dir_count = 0;
 
     for (i = 0; i < panel->dir.len; i++)
     {
@@ -1254,31 +1292,21 @@ panel_compute_totals (const WPanel * panel, void *ui, compute_dir_size_callback
         {
             vfs_path_t *p;
             FileProgressStatus status;
-            size_t lc_dir_count = 0;
-            size_t lc_marked = 0;
-            uintmax_t lc_total2 = 0;
 
             p = vfs_path_append_new (panel->cwd_vpath, panel->dir.list[i].fname, NULL);
-            status = compute_dir_size (p, ui, cback, &lc_dir_count, &lc_marked, &lc_total2,
-                                       compute_symlinks);
+            status = compute_dir_size (p, sm, &dir_count, ret_count, ret_total, compute_symlinks);
             vfs_path_free (p);
 
             if (status != FILE_CONT)
                 return status;
-
-            lc_count += lc_marked;
-            lc_total += lc_total2;
         }
         else
         {
-            lc_count++;
-            lc_total += (uintmax_t) s->st_size;
+            (*ret_count)++;
+            *ret_total += (uintmax_t) s->st_size;
         }
     }
 
-    *ret_count = lc_count;
-    *ret_total = lc_total;
-
     return FILE_CONT;
 }
 
@@ -1298,16 +1326,18 @@ panel_operate_init_totals (const WPanel * panel, const char *source, file_op_con
 
     if (verbose && file_op_compute_totals)
     {
-        ComputeDirSizeUI *ui;
+        dirsize_status_msg_t dsm;
 
-        ui = compute_dir_size_create_ui (TRUE);
+        memset (&dsm, 0, sizeof (dsm));
+        dsm.allow_skip = TRUE;
+        status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 0, dirsize_status_init_cb,
+                         dirsize_status_update_cb, dirsize_status_deinit_cb);
 
         ctx->progress_count = 0;
         ctx->progress_bytes = 0;
 
         if (source == NULL)
-            status = panel_compute_totals (panel, ui, compute_dir_size_update_ui,
-                                           &ctx->progress_count, &ctx->progress_bytes,
+            status = panel_compute_totals (panel, &dsm, &ctx->progress_count, &ctx->progress_bytes,
                                            ctx->follow_links);
         else
         {
@@ -1315,13 +1345,12 @@ panel_operate_init_totals (const WPanel * panel, const char *source, file_op_con
             size_t dir_count = 0;
 
             p = vfs_path_from_str (source);
-            status = compute_dir_size (p, ui, compute_dir_size_update_ui, &dir_count,
-                                       &ctx->progress_count, &ctx->progress_bytes,
-                                       ctx->follow_links);
+            status = compute_dir_size (p, &dsm, &dir_count, &ctx->progress_count,
+                                       &ctx->progress_bytes, ctx->follow_links);
             vfs_path_free (p);
         }
 
-        compute_dir_size_destroy_ui (ui);
+        status_msg_deinit (STATUS_MSG (&dsm));
 
         ctx->progress_totals_computed = (status == FILE_CONT);
 
@@ -1344,19 +1373,16 @@ panel_operate_init_totals (const WPanel * panel, const char *source, file_op_con
 /* --------------------------------------------------------------------------------------------- */
 /**
  * Generate user prompt for panel operation.
- * single_source is the name if the source entry or NULL for multiple
- * entries.
- * src_stat is only used when single_source is not NULL.
+ * src_stat must be not NULL for single source, and NULL for multiple sources
  */
 
 static char *
 panel_operate_generate_prompt (const WPanel * panel, FileOperation operation,
-                               gboolean single_source, const struct stat *src_stat)
+                               const struct stat *src_stat)
 {
-    const char *sp, *cp;
-    char format_string[BUF_MEDIUM];
-    char *dp = format_string;
-    gboolean build_question = FALSE;
+    char *sp;
+    char *format_string;
+    const char *cp;
 
     static gboolean i18n_flag = FALSE;
     if (!i18n_flag)
@@ -1372,86 +1398,61 @@ panel_operate_generate_prompt (const WPanel * panel, FileOperation operation,
 
         one_format = _(one_format);
         many_format = _(many_format);
-        question_format = _(question_format);
 #endif /* ENABLE_NLS */
         i18n_flag = TRUE;
     }
 
-    sp = single_source ? one_format : many_format;
+    /* Possible prompts:
+     *   OP_COPY:
+     *       "Copy file \"%s\" with source mask:"
+     *       "Copy %d files with source mask:"
+     *       "Copy directory \"%s\" with source mask:"
+     *       "Copy %d directories with source mask:"
+     *       "Copy %d files/directories with source mask:"
+     *   OP_MOVE:
+     *       "Move file \"%s\" with source mask:"
+     *       "Move %d files with source mask:"
+     *       "Move directory \"%s\" with source mask:"
+     *       "Move %d directories with source mask:"
+     *       "Move %d files/directories with source mask:"
+     *   OP_DELETE:
+     *       "Delete file \"%s\"?"
+     *       "Delete %d files?"
+     *       "Delete directory \"%s\"?"
+     *       "Delete %d directories?"
+     *       "Delete %d files/directories?"
+     */
 
-    while (*sp != '\0')
-    {
-        switch (*sp)
-        {
-        case '%':
-            cp = NULL;
-            switch (sp[1])
-            {
-            case 'o':
-                cp = op_names1[operation];
-                break;
-            case 'm':
-                if (operation == OP_DELETE)
-                {
-                    cp = "";
-                    build_question = TRUE;
-                }
-                else
-                    cp = prompt_parts[5];
-                break;
-            case 'e':
-                if (operation == OP_DELETE)
-                {
-                    cp = "";
-                    build_question = TRUE;
-                }
-                else
-                    cp = prompt_parts[6];
-                break;
-            case 'f':
-                if (single_source)
-                    cp = S_ISDIR (src_stat->st_mode) ? prompt_parts[2] : prompt_parts[0];
-                else
-                    cp = (panel->marked == panel->dirs_marked)
-                        ? prompt_parts[3]
-                        : (panel->dirs_marked ? prompt_parts[4] : prompt_parts[1]);
-                break;
-            default:
-                *dp++ = *sp++;
-            }
+    sp = (char *) (src_stat != NULL ? one_format : many_format);
 
-            if (cp != NULL)
-            {
-                sp += 2;
+    /* 1. Substitute %o */
+    format_string = str_replace_all (sp, "%o", op_names1[(int) operation]);
 
-                while (*cp != '\0')
-                    *dp++ = *cp++;
+    /* 2. Substitute %n */
+    cp = operation == OP_DELETE ? "\n" : " ";
+    sp = format_string;
+    format_string = str_replace_all (sp, "%n", cp);
+    g_free (sp);
 
-                /* form two-lines query prompt for file deletion */
-                if (operation == OP_DELETE && sp[-1] == 'f')
-                {
-                    *dp++ = '\n';
+    /* 3. Substitute %f */
+    if (src_stat != NULL)
+        cp = S_ISDIR (src_stat->st_mode) ? prompt_parts[2] : prompt_parts[0];
+    else if (panel->marked == panel->dirs_marked)
+        cp = prompt_parts[3];
+    else
+        cp = panel->dirs_marked != 0 ? prompt_parts[4] : prompt_parts[1];
 
-                    while (isblank (*sp) != 0)
-                        sp++;
-                }
-            }
-            break;
-        default:
-            *dp++ = *sp++;
-        }
-    }
-    *dp = '\0';
+    sp = format_string;
+    format_string = str_replace_all (sp, "%f", cp);
+    g_free (sp);
 
-    if (build_question)
-    {
-        char tmp[BUF_MEDIUM];
+    /* 4. Substitute %m */
+    cp = operation == OP_DELETE ? "?" : prompt_parts[5];
+    sp = format_string;
+    format_string = str_replace_all (sp, "%m", cp);
+    g_free (sp);
 
-        memmove (tmp, format_string, sizeof (tmp));
-        g_snprintf (format_string, sizeof (format_string), question_format, tmp);
-    }
-
-    return g_strdup (format_string);
+    return format_string;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1477,7 +1478,7 @@ end_bg_process (file_op_context_t * ctx, enum OperationMode mode)
 /* --------------------------------------------------------------------------------------------- */
 
 FileProgressStatus
-copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
+copy_file_file (file_op_total_context_t * tctx, file_op_context_t * ctx,
                 const char *src_path, const char *dst_path)
 {
     uid_t src_uid = (uid_t) (-1);
@@ -1496,7 +1497,6 @@ copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
     int open_flags;
     gboolean is_first_time = TRUE;
     vfs_path_t *src_vpath = NULL, *dst_vpath = NULL;
-    gboolean write_errno_nospace = FALSE;
 
     /* FIXME: We should not be using global variables! */
     ctx->do_reget = 0;
@@ -1738,28 +1738,39 @@ copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
         goto ret;
     }
 
-    while (TRUE)
+    /* try preallocate space; if fail, try copy anyway */
+    while (vfs_preallocate (dest_desc, file_size, ctx->do_append != 0 ? sb.st_size : 0) != 0)
     {
-        errno = vfs_preallocate (dest_desc, file_size, (ctx->do_append != 0) ? sb.st_size : 0);
-        if (errno == 0)
-            break;
-
         if (ctx->skip_all)
-            return_status = FILE_SKIPALL;
-        else
         {
-            return_status =
-                file_error (_("Cannot preallocate space for target file \"%s\"\n%s"), dst_path);
-            if (return_status == FILE_RETRY)
-                continue;
-            if (return_status == FILE_SKIPALL)
-                ctx->skip_all = TRUE;
+            /* cannot allocate, start the file copying anyway */
+            return_status = FILE_CONT;
+            break;
         }
-        mc_close (dest_desc);
-        dest_desc = -1;
-        mc_unlink (dst_vpath);
-        dst_status = DEST_NONE;
-        goto ret;
+
+        return_status =
+            file_error (_("Cannot preallocate space for target file \"%s\"\n%s"), dst_path);
+
+        if (return_status == FILE_SKIPALL)
+            ctx->skip_all = TRUE;
+
+        if (ctx->skip_all || return_status == FILE_SKIP)
+        {
+            /* skip the space allocation error, start file copying */
+            return_status = FILE_CONT;
+            break;
+        }
+
+        if (return_status == FILE_ABORT)
+        {
+            mc_close (dest_desc);
+            dest_desc = -1;
+            mc_unlink (dst_vpath);
+            dst_status = DEST_NONE;
+            goto ret;
+        }
+
+        /* return_status == FILE_RETRY -- try allocate space again */
     }
 
     ctx->eta_secs = 0.0;
@@ -1821,6 +1832,8 @@ copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
                 /* dst_write */
                 while ((n_written = mc_write (dest_desc, t, n_read)) < n_read)
                 {
+                    gboolean write_errno_nospace;
+
                     if (n_written > 0)
                     {
                         n_read -= n_written;
@@ -1850,10 +1863,6 @@ copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
                     }
                     if (return_status != FILE_RETRY)
                         goto ret;
-
-                    /* User pressed "Retry". Will the next mc_write() call be successful?
-                     * Reset error flag to be ready for that. */
-                    write_errno_nospace = FALSE;
                 }
             }
 
@@ -1932,16 +1941,9 @@ copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
 
     if (dst_status == DEST_SHORT)
     {
-        /* Remove short file */
-        int result = 0;
-
-        /* In case of copy/move to full partition, keep source file
-         * and remove incomplete destination one */
-        if (!write_errno_nospace)
-            result = query_dialog (Q_ ("DialogTitle|Copy"),
-                                   _("Incomplete file was retrieved. Keep it?"),
-                                   D_ERROR, 2, _("&Delete"), _("&Keep"));
-        if (result == 0)
+        /* Query to remove short file */
+        if (query_dialog (Q_ ("DialogTitle|Copy"), _("Incomplete file was retrieved. Keep it?"),
+                          D_ERROR, 2, _("&Delete"), _("&Keep")) == 0)
             mc_unlink (dst_vpath);
     }
     else if (dst_status == DEST_FULL)
@@ -2013,7 +2015,7 @@ copy_file_file (FileOpTotalContext * tctx, file_op_context_t * ctx,
    function calls */
 
 FileProgressStatus
-copy_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s, const char *d,
+copy_dir_dir (file_op_total_context_t * tctx, file_op_context_t * ctx, const char *s, const char *d,
               gboolean toplevel, gboolean move_over, gboolean do_delete, GSList * parent_dirs)
 {
     struct dirent *next;
@@ -2283,10 +2285,10 @@ copy_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
 /* {{{ Move routines */
 
 FileProgressStatus
-move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s, const char *d)
+move_dir_dir (file_op_total_context_t * tctx, file_op_context_t * ctx, const char *s, const char *d)
 {
-    struct stat sbuf, dbuf, destbuf;
-    FileProgressStatus return_status;
+    struct stat sbuf, dbuf;
+    FileProgressStatus return_status = FILE_CONT;
     gboolean move_over = FALSE;
     gboolean dstat_ok;
     vfs_path_t *src_vpath, *dst_vpath;
@@ -2331,7 +2333,7 @@ move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
 
     /* Check if the user inputted an existing dir */
   retry_dst_stat:
-    if (mc_stat (dst_vpath, &destbuf) == 0)
+    if (mc_stat (dst_vpath, &dbuf) == 0)
     {
         if (move_over)
         {
@@ -2345,7 +2347,7 @@ move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
             return_status = FILE_SKIPALL;
         else
         {
-            if (S_ISDIR (destbuf.st_mode))
+            if (S_ISDIR (dbuf.st_mode))
                 return_status = file_error (_("Cannot overwrite directory \"%s\"\n%s"), d);
             else
                 return_status = file_error (_("Cannot overwrite file \"%s\"\n%s"), d);
@@ -2384,6 +2386,7 @@ move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
         goto ret;
   oktoret:
     file_progress_show_source (ctx, NULL);
+    file_progress_show_target (ctx, NULL);
     file_progress_show (ctx, 0, 0, "", FALSE);
 
     return_status = check_progress_buttons (ctx);
@@ -2393,6 +2396,9 @@ move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
     mc_refresh ();
     if (ctx->erase_at_end)
     {
+        /* Reset progress count before delete to avoid counting files twice */
+        tctx->progress_count = tctx->prev_progress_count;
+
         while (erase_list != NULL && return_status != FILE_ABORT)
         {
             struct link *lp = (struct link *) erase_list->data;
@@ -2405,6 +2411,9 @@ move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
             erase_list = g_slist_remove (erase_list, lp);
             free_link (lp);
         }
+
+        /* Save progress counter before move next directory */
+        tctx->prev_progress_count = tctx->progress_count;
     }
     erase_dir_iff_empty (ctx, src_vpath, tctx->progress_count);
 
@@ -2422,7 +2431,7 @@ move_dir_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const char *s,
 /* {{{ Erase routines */
 
 FileProgressStatus
-erase_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const vfs_path_t * s_vpath)
+erase_dir (file_op_total_context_t * tctx, file_op_context_t * ctx, const vfs_path_t * s_vpath)
 {
     FileProgressStatus error;
 
@@ -2464,102 +2473,69 @@ erase_dir (FileOpTotalContext * tctx, file_op_context_t * ctx, const vfs_path_t
 /* --------------------------------------------------------------------------------------------- */
 /* {{{ Panel operate routines */
 
-ComputeDirSizeUI *
-compute_dir_size_create_ui (gboolean allow_skip)
+void
+dirsize_status_init_cb (status_msg_t * sm)
 {
-    ComputeDirSizeUI *ui;
+    dirsize_status_msg_t *dsm = (dirsize_status_msg_t *) sm;
+    Widget *wd = WIDGET (sm->dlg);
 
     const char *b1_name = N_("&Abort");
     const char *b2_name = N_("&Skip");
-    int b1_width, b2_width = 0, b_width = 0;
-    int b1_x;
-    int ui_width;
-    Widget *b;
+    int b_width, ui_width;
 
 #ifdef ENABLE_NLS
     b1_name = _(b1_name);
     b2_name = _(b2_name);
 #endif
 
-    b1_width = str_term_width1 (b1_name) + 4;
-    if (allow_skip)
-        b2_width = str_term_width1 (b2_name) + 4 + 1;
-    b_width = b1_width + b2_width;
-
-    ui = g_new (ComputeDirSizeUI, 1);
+    b_width = str_term_width1 (b1_name) + 4;
+    if (dsm->allow_skip)
+        b_width += str_term_width1 (b2_name) + 4 + 1;
 
     ui_width = max (COLS / 2, b_width + 6);
-    ui->dlg = dlg_create (TRUE, 0, 0, 8, ui_width, dialog_colors, NULL, NULL, NULL,
-                          _("Directory scanning"), DLG_CENTER);
+    dsm->dirname = label_new (2, 3, "");
+    add_widget (sm->dlg, dsm->dirname);
+    dsm->count_size = label_new (3, 3, "");
+    add_widget (sm->dlg, dsm->count_size);
+    add_widget (sm->dlg, hline_new (4, -1, -1));
 
-    ui->dirname = label_new (2, 3, "");
-    add_widget (ui->dlg, ui->dirname);
-    add_widget (ui->dlg, hline_new (4, -1, -1));
-    b1_x = (ui_width - b_width) / 2;
-    b = WIDGET (button_new (5, b1_x, FILE_ABORT, NORMAL_BUTTON, b1_name, NULL));
-    add_widget (ui->dlg, b);
-    if (allow_skip)
+    dsm->abort_button = WIDGET (button_new (5, 3, FILE_ABORT, NORMAL_BUTTON, b1_name, NULL));
+    add_widget (sm->dlg, dsm->abort_button);
+    if (dsm->allow_skip)
     {
-        add_widget (ui->dlg,
-                    button_new (5, b1_x + 1 + b1_width, FILE_SKIP, NORMAL_BUTTON, b2_name, NULL));
-        dlg_select_widget (b);
+        dsm->skip_button = WIDGET (button_new (5, 3, FILE_SKIP, NORMAL_BUTTON, b2_name, NULL));
+        add_widget (sm->dlg, dsm->skip_button);
+        dlg_select_widget (dsm->skip_button);
     }
 
-    /* We will manage the dialog without any help,
-       that's why we have to call dlg_init */
-    dlg_init (ui->dlg);
-
-    return ui;
+    widget_set_size (wd, wd->y, wd->x, 8, ui_width);
+    dirsize_status_locate_buttons (dsm);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-compute_dir_size_destroy_ui (ComputeDirSizeUI * ui)
+int
+dirsize_status_update_cb (status_msg_t * sm)
 {
-    if (ui != NULL)
+    dirsize_status_msg_t *dsm = (dirsize_status_msg_t *) sm;
+    Widget *wd = WIDGET (sm->dlg);
+
+    /* update second (longer label) */
+    label_set_textv (dsm->count_size, _("Directories: %zd, total size: %s"),
+                     dsm->dir_count, size_trunc_sep (dsm->total_size, panels_options.kilobyte_si));
+
+    /* enlarge dialog if required */
+    if (WIDGET (dsm->count_size)->cols + 6 > wd->cols)
     {
-        /* schedule to update passive panel */
-        other_panel->dirty = 1;
-
-        /* close and destroy dialog */
-        dlg_run_done (ui->dlg);
-        dlg_destroy (ui->dlg);
-        g_free (ui);
+        dlg_set_size (sm->dlg, wd->lines, WIDGET (dsm->count_size)->cols + 6);
+        dirsize_status_locate_buttons (dsm);
+        dlg_redraw (sm->dlg);
     }
-}
 
-/* --------------------------------------------------------------------------------------------- */
+    /* adjust first label */
+    label_set_text (dsm->dirname, str_trunc (vfs_path_as_str (dsm->dirname_vpath), wd->cols - 6));
 
-FileProgressStatus
-compute_dir_size_update_ui (void *ui, const vfs_path_t * dirname_vpath, size_t dir_count,
-                            uintmax_t total_size)
-{
-    ComputeDirSizeUI *this = (ComputeDirSizeUI *) ui;
-    int c;
-    Gpm_Event event;
-    char buffer[BUF_1K];
-
-    if (ui == NULL)
-        return FILE_CONT;
-
-    g_snprintf (buffer, sizeof (buffer), _("%s\nDirectories: %zd, total size: %s"),
-                str_trunc (vfs_path_as_str (dirname_vpath), WIDGET (this->dlg)->cols - 6),
-                dir_count, size_trunc_sep (total_size, panels_options.kilobyte_si));
-    label_set_text (this->dirname, buffer);
-
-    event.x = -1;               /* Don't show the GPM cursor */
-    c = tty_get_event (&event, FALSE, FALSE);
-    if (c == EV_NONE)
-        return FILE_CONT;
-
-    /* Reinitialize to avoid old values after events other than
-       selecting a button */
-    this->dlg->ret_value = FILE_CONT;
-
-    dlg_process_event (this->dlg, c, &event);
-
-    switch (this->dlg->ret_value)
+    switch (status_msg_common_update (sm))
     {
     case B_CANCEL:
     case FILE_ABORT:
@@ -2571,6 +2547,17 @@ compute_dir_size_update_ui (void *ui, const vfs_path_t * dirname_vpath, size_t d
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+void
+dirsize_status_deinit_cb (status_msg_t * sm)
+{
+    (void) sm;
+
+    /* schedule to update passive panel */
+    other_panel->dirty = 1;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /**
  * compute_dir_size:
@@ -2579,12 +2566,12 @@ compute_dir_size_update_ui (void *ui, const vfs_path_t * dirname_vpath, size_t d
  */
 
 FileProgressStatus
-compute_dir_size (const vfs_path_t * dirname_vpath, void *ui, compute_dir_size_callback cback,
+compute_dir_size (const vfs_path_t * dirname_vpath, dirsize_status_msg_t * sm,
                   size_t * ret_dir_count, size_t * ret_marked_count, uintmax_t * ret_total,
                   gboolean compute_symlinks)
 {
-    return do_compute_dir_size (dirname_vpath, ui, cback, ret_dir_count, ret_marked_count,
-                                ret_total, compute_symlinks);
+    return do_compute_dir_size (dirname_vpath, sm, ret_dir_count, ret_marked_count, ret_total,
+                                compute_symlinks);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -2623,7 +2610,7 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
     int i;
     FileProgressStatus value;
     file_op_context_t *ctx;
-    FileOpTotalContext *tctx;
+    file_op_total_context_t *tctx;
     vfs_path_t *tmp_vpath;
     filegui_dialog_type_t dialog_type = FILEGUI_DIALOG_ONE_ITEM;
 
@@ -2701,7 +2688,7 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
          * dir is deleted)
          */
         if (!force_single && tmp_dest_dir[0] != '\0'
-            && tmp_dest_dir[strlen (tmp_dest_dir) - 1] != PATH_SEP)
+            && !IS_PATH_SEP (tmp_dest_dir[strlen (tmp_dest_dir) - 1]))
         {
             /* add trailing separator */
             dest_dir = g_strconcat (tmp_dest_dir, PATH_SEP_STR, (char *) NULL);
@@ -2719,7 +2706,8 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
         }
 
         /* Generate confirmation prompt */
-        format = panel_operate_generate_prompt (panel, operation, source != NULL, &src_stat);
+        format =
+            panel_operate_generate_prompt (panel, operation, source != NULL ? &src_stat : NULL);
 
         dest = file_mask_dialog (ctx, operation, source != NULL, format,
                                  source != NULL ? (void *) source
@@ -2742,7 +2730,8 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
         char fmd_buf[BUF_MEDIUM];
 
         /* Generate confirmation prompt */
-        format = panel_operate_generate_prompt (panel, OP_DELETE, source != NULL, &src_stat);
+        format =
+            panel_operate_generate_prompt (panel, OP_DELETE, source != NULL ? &src_stat : NULL);
 
         if (source == NULL)
             g_snprintf (fmd_buf, sizeof (fmd_buf), format, panel->marked);
@@ -2977,7 +2966,7 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
                         value = transform_error;
                     else
                     {
-                        char *temp2, *temp3, *repl_dest, *source_with_path_str;
+                        char *temp2, *repl_dest, *source_with_path_str;
 
                         repl_dest = mc_search_prepare_replace_str2 (ctx->search_handle, dest);
                         if (ctx->search_handle->error != MC_SEARCH_E_OK)
@@ -2992,9 +2981,8 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
                         g_free (repl_dest);
                         source_with_path_str =
                             strutils_shell_unescape (vfs_path_as_str (source_with_vpath));
-                        temp3 = temp2;
-                        temp2 = strutils_shell_unescape (temp2);
-                        g_free (temp3);
+                        temp = strutils_shell_unescape (temp2);
+                        g_free (temp2);
 
                         switch (operation)
                         {
@@ -3008,18 +2996,18 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
                                 vfs_path_free (vpath);
                             }
                             if (S_ISDIR (src_stat.st_mode))
-                                value = copy_dir_dir (tctx, ctx, source_with_path_str, temp2,
+                                value = copy_dir_dir (tctx, ctx, source_with_path_str, temp,
                                                       TRUE, FALSE, FALSE, NULL);
                             else
-                                value = copy_file_file (tctx, ctx, source_with_path_str, temp2);
+                                value = copy_file_file (tctx, ctx, source_with_path_str, temp);
                             dest_dirs = free_linklist (dest_dirs);
                             break;
 
                         case OP_MOVE:
                             if (S_ISDIR (src_stat.st_mode))
-                                value = move_dir_dir (tctx, ctx, source_with_path_str, temp2);
+                                value = move_dir_dir (tctx, ctx, source_with_path_str, temp);
                             else
-                                value = move_file_file (tctx, ctx, source_with_path_str, temp2);
+                                value = move_file_file (tctx, ctx, source_with_path_str, temp);
                             break;
 
                         default:
@@ -3027,7 +3015,7 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
                             abort ();
                         }
 
-                        g_free (temp2);
+                        g_free (temp);
                     }
                 }               /* Copy or move operation */
 
@@ -3079,8 +3067,7 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
 #endif /* WITH_FULL_PATHS */
     g_free (dest);
     vfs_path_free (dest_vpath);
-    g_free (ctx->dest_mask);
-    ctx->dest_mask = NULL;
+    MC_PTR_FREE (ctx->dest_mask);
 
 #ifdef ENABLE_BACKGROUND
     /* Let our parent know we are saying bye bye */
