@@ -2,37 +2,35 @@
    Internal file viewer for the Midnight Commander
    Callback function for some actions (hotkeys, menu)
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
-   Written by: 1994, 1995, 1998 Miguel de Icaza
-   1994, 1995 Janne Kukonlehto
-   1995 Jakub Jelinek
-   1996 Joseph M. Hinkle
-   1997 Norbert Warmuth
-   1998 Pavel Machek
-   2004 Roland Illig <roland.illig@gmx.de>
-   2005 Roland Illig <roland.illig@gmx.de>
-   2009 Slava Zanko <slavazanko@google.com>
-   2009 Andrew Borodin <aborodin@vmail.ru>
-   2009 Ilia Maslakov <il.smind@gmail.com>
+   Written by:
+   Miguel de Icaza, 1994, 1995, 1998
+   Janne Kukonlehto, 1994, 1995
+   Jakub Jelinek, 1995
+   Joseph M. Hinkle, 1996
+   Norbert Warmuth, 1997
+   Pavel Machek, 1998
+   Roland Illig <roland.illig@gmx.de>, 2004, 2005
+   Slava Zanko <slavazanko@google.com>, 2009, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2013
+   Ilia Maslakov <il.smind@gmail.com>, 2009
 
    This file is part of the Midnight Commander.
 
-   The Midnight Commander is free software; you can redistribute it
+   The Midnight Commander is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
 
-   The Midnight Commander is distributed in the hope that it will be
-   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   The Midnight Commander is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -52,22 +50,25 @@
 #include "lib/global.h"
 
 #include "lib/tty/tty.h"
-#include "lib/tty/key.h"
+#include "lib/tty/key.h"        /* is_idle() */
+#include "lib/lock.h"           /* lock_file() */
+#include "lib/util.h"
+#include "lib/widget.h"
+#ifdef HAVE_CHARSET
+#include "lib/charsets.h"
+#endif
+#include "lib/event.h"          /* mc_event_raise() */
+
+#include "src/filemanager/layout.h"
+#include "src/filemanager/cmd.h"
+#include "src/filemanager/midnight.h"   /* current_panel */
+#include "src/filemanager/ext.h"        /* regex_command_for() */
 
-#include "src/dialog.h"         /* cb_ret_t */
-#include "src/panel.h"
-#include "src/layout.h"
-#include "src/wtools.h"
 #include "src/history.h"
-#include "src/charsets.h"
-#include "src/cmd.h"
 #include "src/execute.h"
-#include "src/help.h"
-#include "src/keybind.h"
-#include "src/cmddef.h"         /* CK_ cmd name const */
+#include "src/keybind-defaults.h"
 
 #include "internal.h"
-#include "mcviewer.h"
 
 /*** global variables ****************************************************************************/
 
@@ -77,14 +78,53 @@
 
 /*** file scope variables ************************************************************************/
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+mcview_remove_ext_script (mcview_t * view)
+{
+    if (view->ext_script != NULL)
+    {
+        mc_unlink (view->ext_script);
+        vfs_path_free (view->ext_script);
+        view->ext_script = NULL;
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
 
 /* Both views */
 static void
-mcview_search (mcview_t * view)
+mcview_search (mcview_t * view, gboolean start_search)
 {
-    if (mcview_dialog_search (view))
-        mcview_do_search (view);
+    off_t want_search_start = view->search_start;
+
+    if (start_search)
+    {
+        if (mcview_dialog_search (view))
+        {
+            if (view->hex_mode)
+                want_search_start = view->hex_cursor;
+
+            mcview_do_search (view, want_search_start);
+        }
+    }
+    else
+    {
+        if (view->hex_mode)
+        {
+            if (!mcview_search_options.backwards)
+                want_search_start = view->hex_cursor + 1;
+            else if (view->hex_cursor > 0)
+                want_search_start = view->hex_cursor - 1;
+            else
+                want_search_start = 0;
+        }
+
+        mcview_do_search (view, want_search_start);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -93,180 +133,57 @@ static void
 mcview_continue_search_cmd (mcview_t * view)
 {
     if (view->last_search_string != NULL)
-    {
-        mcview_do_search (view);
-    }
+        mcview_search (view, FALSE);
     else
     {
         /* find last search string in history */
         GList *history;
+
         history = history_get (MC_HISTORY_SHARED_SEARCH);
         if (history != NULL && history->data != NULL)
         {
             view->last_search_string = (gchar *) g_strdup (history->data);
             history = g_list_first (history);
-            g_list_foreach (history, (GFunc) g_free, NULL);
-            g_list_free (history);
+            g_list_free_full (history, g_free);
 
-            view->search = mc_search_new (view->last_search_string, -1);
+#ifdef HAVE_CHARSET
+            view->search = mc_search_new (view->last_search_string, -1, cp_source);
+#else
+            view->search = mc_search_new (view->last_search_string, -1, NULL);
+#endif
             view->search_nroff_seq = mcview_nroff_seq_new (view);
 
-            if (!view->search)
+            if (view->search == NULL)
             {
                 /* if not... then ask for an expression */
-                g_free (view->last_search_string);
-                view->last_search_string = NULL;
-                mcview_search (view);
+                MC_PTR_FREE (view->last_search_string);
+                mcview_search (view, TRUE);
             }
             else
             {
-                view->search->search_type = view->search_type;
-                view->search->is_all_charsets = view->search_all_codepages;
-                view->search->is_case_sentitive = view->search_case;
+                view->search->search_type = mcview_search_options.type;
+#ifdef HAVE_CHARSET
+                view->search->is_all_charsets = mcview_search_options.all_codepages;
+#endif
+                view->search->is_case_sensitive = mcview_search_options.case_sens;
+                view->search->whole_words = mcview_search_options.whole_words;
                 view->search->search_fn = mcview_search_cmd_callback;
                 view->search->update_fn = mcview_search_update_cmd_callback;
-                view->search->whole_words = view->whole_words;
 
-                mcview_do_search (view);
+                mcview_search (view, FALSE);
             }
         }
         else
         {
             /* if not... then ask for an expression */
-            g_free (view->last_search_string);
-            view->last_search_string = NULL;
-            mcview_search (view);
+            MC_PTR_FREE (view->last_search_string);
+            mcview_search (view, TRUE);
         }
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-/* Check for left and right arrows, possibly with modifiers */
-static cb_ret_t
-mcview_check_left_right_keys (mcview_t * view, int c)
-{
-    if (c == KEY_LEFT)
-    {
-        mcview_move_left (view, 1);
-        return MSG_HANDLED;
-    }
-
-    if (c == KEY_RIGHT)
-    {
-        mcview_move_right (view, 1);
-        return MSG_HANDLED;
-    }
-
-    /* Ctrl with arrows moves by 10 postions in the unwrap mode */
-    if (view->hex_mode || view->text_wrap_mode)
-        return MSG_NOT_HANDLED;
-
-    if (c == (KEY_M_CTRL | KEY_LEFT))
-    {
-        if (view->dpy_text_column >= 10)
-            view->dpy_text_column -= 10;
-        else
-            view->dpy_text_column = 0;
-        view->dirty++;
-        return MSG_HANDLED;
-    }
-
-    if (c == (KEY_M_CTRL | KEY_RIGHT))
-    {
-        if (view->dpy_text_column <= OFFSETTYPE_MAX - 10)
-            view->dpy_text_column += 10;
-        else
-            view->dpy_text_column = OFFSETTYPE_MAX;
-        view->dirty++;
-        return MSG_HANDLED;
-    }
-
-    return MSG_NOT_HANDLED;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-static void
-mcview_cmk_move_up (void *w, int n)
-{
-    mcview_move_up ((mcview_t *) w, n);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-static void
-mcview_cmk_move_down (void *w, int n)
-{
-    mcview_move_down ((mcview_t *) w, n);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-static void
-mcview_cmk_moveto_top (void *w, int n)
-{
-    (void) &n;
-    mcview_moveto_top ((mcview_t *) w);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-static void
-mcview_cmk_moveto_bottom (void *w, int n)
-{
-    (void) &n;
-    mcview_moveto_bottom ((mcview_t *) w);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-static inline void
-mcview_moveto_line_cmd (mcview_t * view)
-{
-    char *answer, *answer_end, prompt[BUF_SMALL];
-    off_t line, col;
-
-    mcview_offset_to_coord (view, &line, &col, view->dpy_start);
-
-    g_snprintf (prompt, sizeof (prompt),
-                _(" The current line number is %lld.\n"
-                  " Enter the new line number:"), (long long) (line + 1));
-    answer = input_dialog (_(" Goto line "), prompt, MC_HISTORY_VIEW_GOTO_LINE, "");
-    if (answer != NULL && answer[0] != '\0')
-    {
-        errno = 0;
-        line = strtoul (answer, &answer_end, 10);
-        if (errno == 0 && *answer_end == '\0' && line >= 1)
-            mcview_moveto (view, line - 1, 0);
-    }
-    g_free (answer);
-}
-
-static inline void
-mcview_moveto_addr_cmd (mcview_t * view)
-{
-    char *line, *error, prompt[BUF_SMALL], prompt_format[BUF_SMALL];
-
-    g_snprintf (prompt_format, sizeof (prompt_format),
-                _(" The current address is %s.\n"
-                  " Enter the new address:"), "0x%08" OFFSETTYPE_PRIX "");
-    g_snprintf (prompt, sizeof (prompt), prompt_format, view->hex_cursor);
-    line = input_dialog (_(" Goto Address "), prompt, MC_HISTORY_VIEW_GOTO_ADDR, "");
-    if ((line != NULL) && (*line != '\0'))
-    {
-        off_t addr;
-        addr = strtoul (line, &error, 0);
-        if ((*error == '\0') && mcview_get_byte (view, addr, NULL))
-            mcview_moveto_offset (view, addr);
-        else
-            message (D_ERROR, _("Warning"), _(" Invalid address "));
-    }
-    g_free (line);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static void
 mcview_hook (void *v)
 {
@@ -291,6 +208,8 @@ mcview_hook (void *v)
     else
         return;
 
+    mcview_done (view);
+    mcview_init (view);
     mcview_load (view, 0, panel->dir.list[panel->selected].fname, 0);
     mcview_display (view);
 }
@@ -302,19 +221,19 @@ mcview_handle_editkey (mcview_t * view, int key)
 {
     struct hexedit_change_node *node;
     int byte_val;
+
     /* Has there been a change at this position? */
     node = view->change_list;
-    while (node && (node->offset != view->hex_cursor))
+    while ((node != NULL) && (node->offset != view->hex_cursor))
         node = node->next;
 
     if (!view->hexview_in_text)
     {
         /* Hex editing */
         unsigned int hexvalue = 0;
+
         if (key >= '0' && key <= '9')
-        {
             hexvalue = 0 + (key - '0');
-        }
         else if (key >= 'A' && key <= 'F')
             hexvalue = 10 + (key - 'A');
         else if (key >= 'a' && key <= 'f')
@@ -322,29 +241,31 @@ mcview_handle_editkey (mcview_t * view, int key)
         else
             return MSG_NOT_HANDLED;
 
-        if (node)
+        if (node != NULL)
             byte_val = node->value;
         else
             mcview_get_byte (view, view->hex_cursor, &byte_val);
 
         if (view->hexedit_lownibble)
-        {
             byte_val = (byte_val & 0xf0) | (hexvalue);
-        }
         else
-        {
             byte_val = (byte_val & 0x0f) | (hexvalue << 4);
-        }
     }
     else
     {
         /* Text editing */
-        if (key < 256 && ((key == '\n') || is_printable (key)))
+        if (key < 256 && key != '\t')
             byte_val = key;
         else
             return MSG_NOT_HANDLED;
     }
-    if (!node)
+
+    if ((view->filename_vpath != NULL)
+        && (*(vfs_path_get_last_path_str (view->filename_vpath)) != '\0')
+        && (view->change_list == NULL))
+        view->locked = lock_file (view->filename_vpath);
+
+    if (node == NULL)
     {
         node = g_new (struct hexedit_change_node, 1);
         node->offset = view->hex_cursor;
@@ -352,17 +273,114 @@ mcview_handle_editkey (mcview_t * view, int key)
         mcview_enqueue_change (&view->change_list, node);
     }
     else
-    {
         node->value = byte_val;
-    }
+
     view->dirty++;
-    mcview_update (view);
     mcview_move_right (view, 1);
+
     return MSG_HANDLED;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
+static void
+mcview_load_next_prev_init (mcview_t * view)
+{
+    if (mc_global.mc_run_mode != MC_RUN_VIEWER)
+    {
+        /* get file list from current panel. Update it each time */
+        view->dir = &current_panel->dir;
+        view->dir_idx = &current_panel->selected;
+    }
+    else if (view->dir == NULL)
+    {
+        /* Run from command line */
+        /* Run 1st time. Load/get directory */
+
+        /* TODO: check mtime of directory to reload it */
+
+        const char *fname;
+        size_t fname_len;
+        int i;
+        dir_sort_options_t sort_op = { FALSE, TRUE, FALSE };
+
+        /* load directory where requested file is */
+        view->dir = g_new0 (dir_list, 1);
+        view->dir_idx = g_new (int, 1);
+
+        dir_list_load (view->dir, view->workdir_vpath, (GCompareFunc) sort_name, &sort_op, NULL);
+
+        fname = x_basename (vfs_path_as_str (view->filename_vpath));
+        fname_len = strlen (fname);
+
+        /* search current file in the list */
+        for (i = 0; i != view->dir->len; i++)
+        {
+            const file_entry_t *fe = &view->dir->list[i];
+
+            if (fname_len == fe->fnamelen && strncmp (fname, fe->fname, fname_len) == 0)
+                break;
+        }
+
+        *view->dir_idx = i;
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+mcview_scan_for_file (mcview_t * view, int direction)
+{
+    int i;
+
+    for (i = *view->dir_idx + direction; i != *view->dir_idx; i += direction)
+    {
+        if (i < 0)
+            i = view->dir->len - 1;
+        if (i == view->dir->len)
+            i = 0;
+        if (!S_ISDIR (view->dir->list[i].st.st_mode))
+            break;
+    }
+
+    *view->dir_idx = i;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+mcview_load_next_prev (mcview_t * view, int direction)
+{
+    dir_list *dir;
+    int *dir_idx;
+    vfs_path_t *vfile;
+    vfs_path_t *ext_script = NULL;
+
+    mcview_load_next_prev_init (view);
+    mcview_scan_for_file (view, direction);
+
+    /* reinit view */
+    dir = view->dir;
+    dir_idx = view->dir_idx;
+    view->dir = NULL;
+    view->dir_idx = NULL;
+    vfile = vfs_path_append_new (view->workdir_vpath, dir->list[*dir_idx].fname, (char *) NULL);
+    mcview_done (view);
+    mcview_remove_ext_script (view);
+    mcview_init (view);
+    if (regex_command_for (view, vfile, "View", &ext_script) == 0)
+        mcview_load (view, NULL, vfs_path_as_str (vfile), 0);
+    vfs_path_free (vfile);
+    view->dir = dir;
+    view->dir_idx = dir_idx;
+    view->ext_script = ext_script;
+
+    view->dpy_bbar_dirty = FALSE;       /* FIXME */
+    view->dirty++;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static cb_ret_t
 mcview_execute_cmd (mcview_t * view, unsigned long command)
 {
@@ -370,119 +388,152 @@ mcview_execute_cmd (mcview_t * view, unsigned long command)
 
     switch (command)
     {
-    case CK_ViewHelp:
-        interactive_display (NULL, "[Internal File Viewer]");
+    case CK_Help:
+        {
+            ev_help_t event_data = { NULL, "[Internal File Viewer]" };
+            mc_event_raise (MCEVENT_GROUP_CORE, "help", &event_data);
+        }
         break;
-    case CK_ViewToggleWrapMode:
+    case CK_WrapMode:
         /* Toggle between wrapped and unwrapped view */
         mcview_toggle_wrap_mode (view);
-        mcview_update (view);   /* FIXME: view->dirty++ ? */
         break;
-    case CK_ViewToggleHexEditMode:
+    case CK_HexEditMode:
         /* Toggle between hexview and hexedit mode */
         mcview_toggle_hexedit_mode (view);
-        mcview_update (view);   /* FIXME: view->dirty++ ? */
         break;
-    case CK_ViewToggleHexMode:
+    case CK_HexMode:
         /* Toggle between hex view and text view */
         mcview_toggle_hex_mode (view);
-        mcview_update (view);   /* FIXME: view->dirty++ ? */
         break;
-    case CK_ViewGoto:
-        if (view->hex_mode)
-            mcview_moveto_addr_cmd (view);
-        else
-            mcview_moveto_line_cmd (view);
-        view->dirty++;
-        mcview_update (view);   /* FIXME: unneeded? */
-        break;
-    case CK_ViewHexEditSave:
+    case CK_Goto:
+        {
+            off_t addr;
+
+            if (mcview_dialog_goto (view, &addr))
+            {
+                if (addr >= 0)
+                    mcview_moveto_offset (view, addr);
+                else
+                {
+                    message (D_ERROR, _("Warning"), _("Invalid value"));
+                    view->dirty++;
+                }
+            }
+            break;
+        }
+    case CK_Save:
         mcview_hexedit_save_changes (view);
         break;
-    case CK_ViewSearch:
-        mcview_search (view);
+    case CK_Search:
+        mcview_search (view, TRUE);
         break;
-    case CK_ViewToggleMagicMode:
+    case CK_SearchForward:
+        mcview_search_options.backwards = FALSE;
+        mcview_search (view, TRUE);
+        break;
+    case CK_SearchBackward:
+        mcview_search_options.backwards = TRUE;
+        mcview_search (view, TRUE);
+        break;
+    case CK_MagicMode:
         mcview_toggle_magic_mode (view);
-        mcview_update (view);   /* FIXME: view->dirty++ ? */
         break;
-    case CK_ViewToggleNroffMode:
+    case CK_NroffMode:
         mcview_toggle_nroff_mode (view);
-        mcview_update (view);   /* FIXME: view->dirty++ ? */
         break;
-    case CK_ViewToggleHexNavMode:
+    case CK_ToggleNavigation:
         view->hexview_in_text = !view->hexview_in_text;
         view->dirty++;
         break;
-    case CK_ViewMoveToBol:
+    case CK_Home:
         mcview_moveto_bol (view);
-        view->dirty++;
         break;
-    case CK_ViewMoveToEol:
+    case CK_End:
         mcview_moveto_eol (view);
         break;
-    case CK_ViewMoveLeft:
+    case CK_Left:
         mcview_move_left (view, 1);
         break;
-    case CK_ViewMoveRight:
+    case CK_Right:
         mcview_move_right (view, 1);
         break;
-        /* Continue search */
-    case CK_ViewContinueSearch:
+    case CK_LeftQuick:
+        if (!view->hex_mode)
+            mcview_move_left (view, 10);
+        break;
+    case CK_RightQuick:
+        if (!view->hex_mode)
+            mcview_move_right (view, 10);
+        break;
+    case CK_SearchContinue:
         mcview_continue_search_cmd (view);
         break;
-    case CK_ViewToggleRuler:
+    case CK_SearchForwardContinue:
+        mcview_search_options.backwards = FALSE;
+        mcview_continue_search_cmd (view);
+        break;
+    case CK_SearchBackwardContinue:
+        mcview_search_options.backwards = TRUE;
+        mcview_continue_search_cmd (view);
+        break;
+    case CK_Ruler:
         mcview_display_toggle_ruler (view);
         break;
-    case CK_ViewMoveUp:
+    case CK_Up:
         mcview_move_up (view, 1);
         break;
-    case CK_ViewMoveDown:
+    case CK_Down:
         mcview_move_down (view, 1);
         break;
-    case CK_ViewMoveHalfPgUp:
+    case CK_HalfPageUp:
         mcview_move_up (view, (view->data_area.height + 1) / 2);
         break;
-    case CK_ViewMoveHalfPgDn:
+    case CK_HalfPageDown:
         mcview_move_down (view, (view->data_area.height + 1) / 2);
         break;
-    case CK_ViewMovePgUp:
+    case CK_PageUp:
         mcview_move_up (view, view->data_area.height);
         break;
-    case CK_ViewMovePgDn:
+    case CK_PageDown:
         mcview_move_down (view, view->data_area.height);
         break;
-    case CK_ShowCommandLine:
+    case CK_Top:
+        mcview_moveto_top (view);
+        break;
+    case CK_Bottom:
+        mcview_moveto_bottom (view);
+        break;
+    case CK_Shell:
         view_other_cmd ();
         break;
-        /*
-           // Unlike Ctrl-O, run a new shell if the subshell is not running
-           case '!':
-           exec_shell ();
-           return MSG_HANDLED;
-         */
-    case CK_ViewGotoBookmark:
+    case CK_BookmarkGoto:
         view->marks[view->marker] = view->dpy_start;
         break;
-    case CK_ViewNewBookmark:
+    case CK_Bookmark:
         view->dpy_start = view->marks[view->marker];
+        view->dpy_paragraph_skip_lines = 0;     /* TODO: remember this value in the marker? */
+        view->dpy_wrap_dirty = TRUE;
         view->dirty++;
         break;
+#ifdef HAVE_CHARSET
     case CK_SelectCodepage:
         mcview_select_encoding (view);
         view->dirty++;
-        mcview_update (view);
         break;
-    case CK_ViewNextFile:
-    case CK_ViewPrevFile:
-        /* Use to indicate parent that we want to see the next/previous file */
+#endif
+    case CK_FileNext:
+    case CK_FilePrev:
         /* Does not work in panel mode */
         if (!mcview_is_in_panel (view))
-            view->move_dir = (command == CK_ViewNextFile) ? 1 : -1;
-        /* fallthrough */
-    case CK_ViewQuit:
-        if (mcview_ok_to_quit (view))
-            view->want_to_quit = TRUE;
+            mcview_load_next_prev (view, command == CK_FileNext ? 1 : -1);
+        break;
+    case CK_Quit:
+        if (!mcview_is_in_panel (view))
+            dlg_stop (WIDGET (view)->owner);
+        break;
+    case CK_Cancel:
+        /* don't close viewer due to SIGINT */
         break;
     default:
         res = MSG_NOT_HANDLED;
@@ -490,34 +541,29 @@ mcview_execute_cmd (mcview_t * view, unsigned long command)
     return res;
 }
 
-/* Both views */
+/* --------------------------------------------------------------------------------------------- */
+/** Both views */
 static cb_ret_t
 mcview_handle_key (mcview_t * view, int key)
 {
     unsigned long command;
 
+#ifdef HAVE_CHARSET
     key = convert_from_input_c (key);
+#endif
 
     if (view->hex_mode)
     {
         if (view->hexedit_mode && (mcview_handle_editkey (view, key) == MSG_HANDLED))
             return MSG_HANDLED;
 
-        command = lookup_keymap_command (view->hex_map, key);
-        if ((command != CK_Ignore_Key) && (mcview_execute_cmd (view, command) == MSG_HANDLED))
+        command = keybind_lookup_keymap_command (viewer_hex_map, key);
+        if ((command != CK_IgnoreKey) && (mcview_execute_cmd (view, command) == MSG_HANDLED))
             return MSG_HANDLED;
     }
 
-    command = lookup_keymap_command (view->plain_map, key);
-    if ((command != CK_Ignore_Key) && (mcview_execute_cmd (view, command) == MSG_HANDLED))
-        return MSG_HANDLED;
-
-    if (mcview_check_left_right_keys (view, key))
-        return MSG_HANDLED;
-
-    if (check_movement_keys (key, view->data_area.height + 1, view,
-                             mcview_cmk_move_up, mcview_cmk_move_down,
-                             mcview_cmk_moveto_top, mcview_cmk_moveto_bottom))
+    command = keybind_lookup_keymap_command (viewer_map, key);
+    if ((command != CK_IgnoreKey) && (mcview_execute_cmd (view, command) == MSG_HANDLED))
         return MSG_HANDLED;
 
 #ifdef MC_ENABLE_DEBUGGING_CODE
@@ -538,7 +584,7 @@ mcview_handle_key (mcview_t * view, int key)
 /* --------------------------------------------------------------------------------------------- */
 
 static inline void
-mcview_adjust_size (Dlg_head * h)
+mcview_adjust_size (WDialog * h)
 {
     mcview_t *view;
     WButtonBar *b;
@@ -547,99 +593,162 @@ mcview_adjust_size (Dlg_head * h)
     view = (mcview_t *) find_widget_type (h, mcview_callback);
     b = find_buttonbar (h);
 
-    widget_set_size (&view->widget, 0, 0, LINES - 1, COLS);
-    widget_set_size (&b->widget, LINES - 1, 0, 1, COLS);
+    widget_set_size (WIDGET (view), 0, 0, LINES - 1, COLS);
+    widget_set_size (WIDGET (b), LINES - 1, 0, 1, COLS);
 
+    view->dpy_wrap_dirty = TRUE;
     mcview_compute_areas (view);
     mcview_update_bytes_per_line (view);
 }
 
-
 /* --------------------------------------------------------------------------------------------- */
 
-/*** public functions ****************************************************************************/
+static gboolean
+mcview_ok_to_quit (mcview_t * view)
+{
+    int r;
 
+    if (view->change_list == NULL)
+        return TRUE;
+
+    if (!mc_global.midnight_shutdown)
+    {
+        query_set_sel (2);
+        r = query_dialog (_("Quit"),
+                          _("File was modified. Save with exit?"), D_NORMAL, 3,
+                          _("&Yes"), _("&No"), _("&Cancel quit"));
+    }
+    else
+    {
+        r = query_dialog (_("Quit"),
+                          _("Midnight Commander is being shut down.\nSave modified file?"),
+                          D_NORMAL, 2, _("&Yes"), _("&No"));
+        /* Esc is No */
+        if (r == -1)
+            r = 1;
+    }
+
+    switch (r)
+    {
+    case 0:                    /* Yes */
+        return mcview_hexedit_save_changes (view) || mc_global.midnight_shutdown;
+    case 1:                    /* No */
+        mcview_hexedit_free_change_list (view);
+        return TRUE;
+    default:
+        return FALSE;
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
 cb_ret_t
-mcview_callback (Widget * w, widget_msg_t msg, int parm)
+mcview_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     mcview_t *view = (mcview_t *) w;
     cb_ret_t i;
-    Dlg_head *h = view->widget.parent;
 
     mcview_compute_areas (view);
     mcview_update_bytes_per_line (view);
 
     switch (msg)
     {
-    case WIDGET_INIT:
+    case MSG_INIT:
         if (mcview_is_in_panel (view))
             add_hook (&select_file_hook, mcview_hook, view);
         else
             view->dpy_bbar_dirty = TRUE;
         return MSG_HANDLED;
 
-    case WIDGET_DRAW:
+    case MSG_DRAW:
         mcview_display (view);
         return MSG_HANDLED;
 
-    case WIDGET_CURSOR:
+    case MSG_CURSOR:
         if (view->hex_mode)
             mcview_place_cursor (view);
         return MSG_HANDLED;
 
-    case WIDGET_KEY:
+    case MSG_KEY:
         i = mcview_handle_key (view, parm);
-        if (view->want_to_quit && !mcview_is_in_panel (view))
-            dlg_stop (h);
-        else
-            mcview_update (view);
-        return i;
+        mcview_update (view);
+        /* don't pass any chars to command line in QuickView mode */
+        return mcview_is_in_panel (view) ? MSG_HANDLED : i;
 
-    case WIDGET_COMMAND:
+    case MSG_ACTION:
         i = mcview_execute_cmd (view, parm);
-        if (view->want_to_quit && !mcview_is_in_panel (view))
-            dlg_stop (h);
-        else
-            mcview_update (view);
+        mcview_update (view);
         return i;
 
-    case WIDGET_FOCUS:
+    case MSG_FOCUS:
+        view->active = TRUE;
         view->dpy_bbar_dirty = TRUE;
         mcview_update (view);
         return MSG_HANDLED;
 
-    case WIDGET_DESTROY:
-        mcview_done (view);
+    case MSG_UNFOCUS:
+        view->active = FALSE;
+        return MSG_HANDLED;
+
+    case MSG_DESTROY:
         if (mcview_is_in_panel (view))
+        {
             delete_hook (&select_file_hook, mcview_hook);
+
+            if (mc_global.midnight_shutdown)
+                mcview_ok_to_quit (view);
+        }
+        mcview_done (view);
+        mcview_remove_ext_script (view);
         return MSG_HANDLED;
 
     default:
-        return default_proc (msg, parm);
+        return widget_default_callback (w, sender, msg, parm, data);
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 cb_ret_t
-mcview_dialog_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *data)
+mcview_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
-    mcview_t *view = data;
+    WDialog *h = DIALOG (w);
+    mcview_t *view;
 
     switch (msg)
     {
-    case DLG_RESIZE:
+    case MSG_RESIZE:
         mcview_adjust_size (h);
         return MSG_HANDLED;
 
-    case DLG_ACTION:
-        /* command from buttonbar */
-        return send_message ((Widget *) view, WIDGET_COMMAND, parm);
+    case MSG_ACTION:
+        /* shortcut */
+        if (sender == NULL)
+            return mcview_execute_cmd (NULL, parm);
+        /* message from buttonbar */
+        if (sender == WIDGET (find_buttonbar (h)))
+        {
+            if (data != NULL)
+                return send_message (data, NULL, MSG_ACTION, parm, NULL);
+
+            view = (mcview_t *) find_widget_type (h, mcview_callback);
+            return mcview_execute_cmd (view, parm);
+        }
+        return MSG_NOT_HANDLED;
+
+    case MSG_VALIDATE:
+        view = (mcview_t *) find_widget_type (h, mcview_callback);
+        h->state = DLG_ACTIVE;  /* don't stop the dialog before final decision */
+        if (mcview_ok_to_quit (view))
+            h->state = DLG_CLOSED;
+        else
+            mcview_update (view);
+        return MSG_HANDLED;
 
     default:
-        return default_dlg_callback (h, sender, msg, parm, data);
+        return dlg_default_callback (w, sender, msg, parm, data);
     }
 }
 
